@@ -237,6 +237,15 @@ def executeSnd(
 
     return df
 
+def fetchApiData(url, headers, program, methode, cono, record):
+
+    data = {"program": program, "cono": cono}
+    data['transactions'] = [{"transaction": methode, "record": record}]
+    
+    #r = controller.sendresults(url, headers, data)
+    r = {'results': [{'transaction': 'StartEntry', 'records': [{'MSGN': '0000000012'}]}], 'wasTerminated': False, 'nrOfSuccessfullTransactions': 1, 'nrOfFailedTransactions': 0}
+    print(r)
+    return r
 
 def getSuccessGraphDataframe(df):
     results = {}
