@@ -10,12 +10,15 @@ import numpy as np
 import pandas as pd
 from inforion.logger.logger import get_logger
 from inforion.transformation.transform_error import TransformationError
+import inforion.ionapi.model.inforlogin as inforlogin
+from inforion.ionapi.model import *
 
 logger = get_logger("transform", True)
+m3_id = ''
 
 
 def parallelize_tranformation(
-    mappingfile, mainsheet, stagingdata, outputfile=None, n_cores=4
+    mappingfile, mainsheet, stagingdata, outputfile=None, url=None, ionfile=None, n_cores=4
 ):
 
     # Read the file from given location
@@ -38,6 +41,9 @@ def parallelize_tranformation(
         stagingdata = ts.handle_special_pre_transformations(
             sheet_to_df_map, mainsheet, stagingdata
         )
+    
+    # Checking for special api calls
+    main_cache = getColumnsApiData(main_cache, url, ionfile)
 
     # Transforming data in parallel
     if n_cores == 1:
@@ -71,6 +77,7 @@ def parallelize_tranformation(
     if outputfile is not None:
         logging.info("Save to file: " + outputfile)
         writer = pd.ExcelWriter(outputfile, engine="xlsxwriter")
+        writer.book.use_zip64()
         df.to_excel(writer, sheet_name="Log Output", index=False)
         writer.save()
     return df
@@ -152,6 +159,42 @@ def getTabsMappingCache(sheet_to_df_map, mapping_cache):
 
     return mapping_sheets_cache
 
+def getColumnsApiData(mapping_cache, url, ionfile):
+    """Get additional information from API"""
+    cono = 409
+    for map in mapping_cache:
+        if map["API_FIELD"] == "CONO" and map["FUNC_VAL"] != "": # get company number if exist in mapping
+            cono = int(map["FUNC_VAL"])
+            
+        if map["FUNC_TYPE"] == "api":
+            api_values = map["FUNC_ARG"].split("|")
+            headers = inforlogin.get_headers_info(ionfile)           
+            data = {api_values[2]:api_values[3], "CONO":cono}
+            program = api_values[0]
+            methode = api_values[1]
+            result = fetchApiData(url, headers, program, methode, cono, data)
+            if len(result) > 0:
+                if "results" in result.keys():
+                    if len(result["results"]) > 0:
+                        for key in result["results"]:
+                            if(result["nrOfSuccessfullTransactions"] == 1 and len(key['records']) > 0):
+                                map["FUNC_TYPE"] = "const"
+                                map["FUNC_VAL"] = key["records"][0]["MSGN"]
+                                map["FUNC_ARG"] = ""
+                            else:
+                                raise TransformationError(
+                                    "Value for the field '{}' mentioned in mapping sheet is not found in M3.".format(
+                                        map["API_FIELD"]
+                                    )
+                                )
+            else:
+                raise TransformationError(
+                    "Value for the field '{}' mentioned in mapping sheet is not found in M3.".format(
+                        map["API_FIELD"]
+                    )
+                )
+                
+    return mapping_cache
 
 def transform_data(
     _sheet_to_df_map, _mainsheet, sheet_cache, tabs_cache, wildcard_tabs, stagingdata
@@ -252,7 +295,7 @@ def transform_sheet_table_mapping_column(
 
 
 def transform_function_column(map_row, map_col, tb_row, func_name, func_args):
-    """Reads the value from source table against source col, and maps it to map_col in map_row"""
+    """Reads the value from api against source col, and maps it to map_col in map_row"""
     if func_name.strip().lower() == "div":
         data_values = func_args.split("|")
         with decimal.localcontext() as ctx:
@@ -264,7 +307,6 @@ def transform_function_column(map_row, map_col, tb_row, func_name, func_args):
 
     map_row[map_col] = result
 
-
 def transform_const_column(map_row, map_col, tb_row, func_val):
     if isinstance(func_val, datetime.datetime):
         val = func_val.strftime("%Y%m%d")
